{% extends "base.html" %}

{% block title %}
    Telegram Login riuscito
{% endblock %}

{% block content %}
    <div class="dbox">
        <div class="dbox-top">
            <span class="left">
                Telegram Login
            </span>
        </div>
        <div class="dbox-bot">
            Login riuscito! Sei connesso come <b>{{ session["royal"]["username"] }}</b>!
        </div>
    </div>
{% endblock %}
